/*
   Copyright Universite de Versailles Saint-Quentin en Yvelines 2009
   AUTHORS: Sebastien Briais, Sid-Ahmed-Ali Touati

   This file is part of SIRA.
   
   SIRA is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.
   
   SIRA is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with SIRA.  If not, see
   <http://www.gnu.org/licenses/>.
*/
#ifndef __ILP_SOLVE_H
#define __ILP_SOLVE_H

/** LP_SOLVE stub */

#include "lp_lib.h"
#include <math.h>

#define ILP_SOLVER_LO (1<<0)
#define ILP_SOLVER_UP (1<<1)
#define ILP_SOLVER_DB (ILP_SOLVER_LO | ILP_SOLVER_UP)

#define ILP_SOLVER_TYPE lprec
#define ILP_SOLVER_COEF_TYPE REAL

static void ilp_solver_init() {
}

static lprec *ilp_solver_create(const char *name, int min_flag, int cols) {
  lprec *prob = make_lp(0, cols);
  if(prob == NULL) {
    return NULL;
  }
  set_verbose(prob, CRITICAL);
  set_lp_name(prob, (char *)name);
  if(min_flag) {
    set_minim(prob);
  } else {
    set_maxim(prob);
  }
  int i;
  for(i = 1; i <= cols; i++) {
    set_int(prob, i, 1);
  }
  return prob;
}

static void ilp_solver_set_col_double(lprec *prob, int i) {
  set_int(prob, i, 0);
}

static void ilp_solver_delete(lprec *prob) {
  delete_lp(prob);
}

static void ilp_solver_cleanup() {
}

static void ilp_solver_set_col_name(lprec *prob, int i, const char *name) {
  if(name != NULL) {
    set_col_name(prob, i, (char *)name);
  }
}

static void ilp_solver_set_col_bounds(lprec *prob, int i, int bnds, REAL lower, REAL upper) {
  if(bnds & ILP_SOLVER_LO) {
    if(bnds & ILP_SOLVER_UP) {
      set_bounds(prob, i, lower, upper);
    } else {
      set_bounds(prob, i, lower, get_infinite(prob));
    }
  } else if(bnds & ILP_SOLVER_UP) {
    set_bounds(prob, i, -get_infinite(prob), upper);
  } else {
    set_unbounded(prob, i);
  }
}

static void ilp_solver_set_obj_coef(lprec *prob, int i, REAL coef) {
  set_obj(prob, i, coef);
}

static int ilp_solver_add_row(lprec *prob, const char *name, int nb, int idx[], REAL coef[], int bnds, REAL bound) {
  coef++;
  idx++;
  if(bnds & ILP_SOLVER_LO) {
    add_constraintex(prob, nb, coef, idx, GE, bound);
  } else if(bnds & ILP_SOLVER_UP) {
    add_constraintex(prob, nb, coef, idx, LE, bound);
  }
  int i = get_Nrows(prob);
  if(name != NULL) {
    set_row_name(prob, i, (char *)name);
  }
  return i;
}

static int ilp_solver_solve(lprec *prob) {
  if(solve(prob) == 0) {
    return TRUE;
  }
  return FALSE;
}

static int ilp_solver_get_col_value_int(lprec *prob, int i) {
  return rint(get_var_primalresult(prob, get_Nrows(prob)+i));
}

static double ilp_solver_get_col_value_double(lprec *prob, int i) {
  return get_var_primalresult(prob, get_Nrows(prob)+i);
}

static REAL ilp_solver_get_obj_value(lprec *prob) {
  return get_var_primalresult(prob, 0);
}

#ifdef DEBUG
static void ilp_solver_write_lp(lprec *prob, const char *name) {
  write_lp(prob, (char *)name);
}
#endif

#endif

