/*
   Copyright Universite de Versailles Saint-Quentin en Yvelines 2009
   AUTHORS: Sebastien Briais, Sid-Ahmed-Ali Touati

   This file is part of SIRA.
   
   SIRA is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.
   
   SIRA is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with SIRA.  If not, see
   <http://www.gnu.org/licenses/>.
*/

#include "sira.h"
#include "reduce.h"
#include "solver.h"
#include "isolver.h"
#include "util.h"

int SIRA_try_reduce_acyclic_saturation(SCEDA_Graph *ddg, SIRA_WeightMap *weight_map, SCEDA_List **added_edges) {
  SCEDA_List *new_edges;
  if(SIRA_reduce(ddg, weight_map, &new_edges) >= 0) {
    *added_edges = new_edges;
    return 0;
  } else {
    return -1;
  }
}

int SIRA_try_reduce_cyclic_saturation(SIRA_Loop *loop, SIRA_Param *settings, SIRA_RegConstraintsMap *constraints, SIRA_WeightMap *weight_map, SCEDA_List **added_edges) {
  SCEDA_HashMap *solution;
  int II = SIRA_solver_search(loop, constraints, settings, weight_map, -1, -1, &solution);
  if(II >= 0) {
    SCEDA_List *new_edges = SCEDA_list_create(NULL);

    SCEDA_HashMapIterator decisions;
    SCEDA_hashmap_iterator_init(solution, &decisions);
    while(SCEDA_hashmap_iterator_has_next(&decisions)) {
      char *type;
      SCEDA_List *reuse_edges = SCEDA_hashmap_iterator_next(&decisions, &type);
      SCEDA_List *antidep_edges = SIRA_reuse_apply_decisions(SIRA_loop_ddg(loop), type, reuse_edges, SIRA_param_get_ual_flag(settings));
      {
	SCEDA_ListIterator edges;
	SCEDA_list_iterator_init(antidep_edges, &edges);
	while(SCEDA_list_iterator_has_next(&edges)) {
	  SCEDA_Edge *e = SCEDA_list_iterator_next(&edges);
	  safe_call(SCEDA_list_add(new_edges, e));
	}
	SCEDA_list_iterator_cleanup(&edges);
      }
      SCEDA_list_delete(antidep_edges);
    }
    SCEDA_hashmap_iterator_cleanup(&decisions);

    SCEDA_hashmap_delete(solution);

    if(SIRA_loop_update(loop) == 0) {
      *added_edges = new_edges;
      return 0;
    } else {
      GDD_delete_edges(SIRA_loop_ddg(loop), new_edges);
      SCEDA_list_delete(new_edges);

      SIRA_loop_update(loop);
      
      return -1;
    }
  } else {
    return -1;
  }
}

int SIRA_try_reduce_cyclic_saturation_iteratively(SIRA_Loop *loop, SIRA_Param *settings, SIRA_RegConstraintsMap *constraints, SCEDA_List *types, SCEDA_List **added_edges) {
  SCEDA_HashMap *solution;
  SCEDA_HashMap *costmap;
  int II = SIRA_isolver_search(loop, constraints, settings, types, -1, -1, &solution, &costmap);
  if(II >= 0) {
    SCEDA_List *new_edges = SCEDA_list_create(NULL);

    SCEDA_HashMapIterator decisions;
    SCEDA_hashmap_iterator_init(solution, &decisions);
    while(SCEDA_hashmap_iterator_has_next(&decisions)) {
      char *type;
      SCEDA_List *antidep_edges = SCEDA_hashmap_iterator_next(&decisions, &type);
      {
	SCEDA_ListIterator edges;
	SCEDA_list_iterator_init(antidep_edges, &edges);
	while(SCEDA_list_iterator_has_next(&edges)) {
	  SCEDA_Edge *e = SCEDA_list_iterator_next(&edges);
	  safe_call(SCEDA_list_add(new_edges, e));
	}
	SCEDA_list_iterator_cleanup(&edges);
      }
    }
    SCEDA_hashmap_iterator_cleanup(&decisions);

    SCEDA_hashmap_delete(solution);
    SCEDA_hashmap_delete(costmap);

    *added_edges = new_edges;
    return 0;
  } else {
    return -1;
  }
}
